#ifndef INCLUDED_RENDER_H
#define INCLUDED_RENDER_H

#define COLOR_WHITE   0x0FFFFFFF
#define COLOR_GREY    0xB2B2B2B2
#define COLOR_BLACK   0x00000000

#define COLOR_RED     0x00000FFF
#define COLOR_GREEN   0x0000FF00
#define COLOR_BLUE    0x00FF0000

#define COLOR_CYAN    0x00FFFF00
#define COLOR_MAGENTA 0x00FF00FF
#define COLOR_YELLOW  0x0000FFFF
#define COLOR_PURPLE  0x00FF0FFF


#define BRIGHTNESS_BRIGHT  1
#define BRIGHTNESS_NORMAL  0
#define BRIGHTNESS_DIM    -1

#define MODIFY_BRIGHT 0xFFFFFFFF
#define MODIFY_NORMAL 0xB2B2B2B2
#define MODIFY_DIM    0x51515151



/* messageBox for the Network/UI Thread to message the Render Thread */
extern SceUID renderMessagebox;

//Renderer Buffer
extern unsigned int __attribute__((aligned(16))) terminal_pixels_full[(512*272) + (512*270)];
extern unsigned int __attribute__((aligned(16))) *terminal_pixels;//[512*272];
extern unsigned int __attribute__((aligned(16))) list[262144];

extern void* framebuffer;

int renderThread(SceSize args, void *argp);

enum {
RENDER_MAIN,
RENDER_RESET,
RENDER_GOTO,
RENDER_GOBY,
RENDER_REDRAW,
RENDER_PUTSTRING,
RENDER_MOVE,
RENDER_MODE,
RENDER_SAVELOAD,
RENDER_CLEAR,
RENDER_SET_REGION,
RENDER_SETFONT,
RENDER_DRAW_CURRENT_POS
};
//Add something to be rendered in the main render area
#define renderMain(string, textColor) {\
	RenderMsg* AMsg = malloc(sizeof(RenderMsg));\
	AMsg->text = strdup(string);\
	AMsg->color = textColor;\
	AMsg->bgcolor = COLOR_BLACK;\
	AMsg->flags = RENDER_MAIN;\
	sceKernelSendMbx(renderMessagebox, AMsg);\
	}

#define renderMainBG(string, textColor, BGColor) {\
	RenderMsg* AMsg = malloc(sizeof(RenderMsg));\
	AMsg->text = strdup(string);\
	AMsg->color = textColor;\
	AMsg->bgcolor = BGColor;\
	AMsg->flags = RENDER_MAIN;\
	sceKernelSendMbx(renderMessagebox, AMsg);\
	}

//Clear the main Render Area
#define renderReset() {\
	RenderMsg* AMsg = malloc(sizeof(RenderMsg));\
	AMsg->flags=RENDER_RESET;\
	sceKernelSendMbx(renderMessagebox, AMsg);\
	}

//Move the cursor to x,y. if either is negative it will be ignored
#define renderGoto(x, y) {\
	RenderMsg* AMsg = malloc(sizeof(RenderMsg));\
	AMsg->flags=RENDER_GOTO;\
	AMsg->extra1=x;\
	AMsg->extra2=y;\
	sceKernelSendMbx(renderMessagebox, AMsg);\
	}

//Move the cursor by x,y. sticks to edges
#define renderGoBy(x, y) {\
	RenderMsg* AMsg = malloc(sizeof(RenderMsg));\
	AMsg->flags=RENDER_GOBY;\
	AMsg->extra1=x;\
	AMsg->extra2=y;\
	AMsg->color=false;\
	sceKernelSendMbx(renderMessagebox, AMsg);\
	}

//Same as above except scrolls the area if at an edge
#define renderGoByMove(x, y) {\
	RenderMsg* AMsg = malloc(sizeof(RenderMsg));\
	AMsg->flags=RENDER_GOBY;\
	AMsg->extra1=x;\
	AMsg->extra2=y;\
	AMsg->color=true;\
	sceKernelSendMbx(renderMessagebox, AMsg);\
	}

//Redraw the Current Screen
#define renderRedraw() {\
	RenderMsg* AMsg = malloc(sizeof(RenderMsg));\
	AMsg->flags=RENDER_REDRAW;\
	sceKernelSendMbx(renderMessagebox, AMsg);\
	}

//Put a string at a position without moving the current Position or scrolling etc...
#define renderPutString(x, y, textColor, string) {\
	RenderMsg* AMsg = malloc(sizeof(RenderMsg));\
	AMsg->text = strdup(string);\
	AMsg->color = textColor;\
	AMsg->bgcolor = COLOR_BLACK;\
	AMsg->extra1=x;\
	AMsg->extra2=y;\
	AMsg->flags=RENDER_PUTSTRING;\
	sceKernelSendMbx(renderMessagebox, AMsg);\
	}
//Put a string at a position without moving the current Position or scrolling etc...
#define renderPutStringBG(x, y, textColor, BGColor, string) {\
	RenderMsg* AMsg = malloc(sizeof(RenderMsg));\
	AMsg->text = strdup(string);\
	AMsg->color = textColor;\
	AMsg->bgcolor = BGColor;\
	AMsg->extra1=x;\
	AMsg->extra2=y;\
	AMsg->flags=RENDER_PUTSTRING;\
	sceKernelSendMbx(renderMessagebox, AMsg);\
	}

//Change the currently rendered area of the screen
#define renderMove(xmov) {\
	RenderMsg* AMsg = malloc(sizeof(RenderMsg));\
	AMsg->flags=RENDER_MOVE;\
	AMsg->extra1=xmov;\
	sceKernelSendMbx(renderMessagebox, AMsg);\
	}

//Change the renderer Mode, slow -- immediate redraw, fast -- redraw only when told
#define renderMode(newmode) {\
	RenderMsg* AMsg = malloc(sizeof(RenderMsg));\
	AMsg->flags=RENDER_MODE;\
	AMsg->extra1=newmode;\
	sceKernelSendMbx(renderMessagebox, AMsg);\
	}
//if RENDER_FAST is set then the buffer must be manually flushed with redraw.
#define RENDER_SLOW 0
#define RENDER_FAST 1


//Make the renderer save/restore the current pos
#define renderSaveLoad(type) {\
	RenderMsg* AMsg = malloc(sizeof(RenderMsg));\
	AMsg->flags=RENDER_SAVELOAD;\
	AMsg->extra1=type;\
	sceKernelSendMbx(renderMessagebox, AMsg);\
	}
#define SAVE 0
#define LOAD 1

//Clear portions of the screen
#define renderClear(type, bgColor) {\
	RenderMsg* AMsg = malloc(sizeof(RenderMsg));\
	AMsg->flags=RENDER_CLEAR;\
	AMsg->color=bgColor;\
	AMsg->extra1=type;\
	sceKernelSendMbx(renderMessagebox, AMsg);\
	}
enum {
CLEAR_TO_EOL,
CLEAR_TO_SOL,
CLEAR_LINE,
CLEAR_LINE_TO_BOTTOM,
CLEAR_LINE_TO_TOP,
CLEAR_SCREEN	//resets current pos to 0,0
};

//Set a render region (behaves like a vt100 scroll region)
#define renderSetRegion(start, end) {\
	RenderMsg* AMsg = malloc(sizeof(RenderMsg));\
	AMsg->flags=RENDER_SET_REGION;\
	AMsg->extra1=start;\
	AMsg->extra2=end;\
	sceKernelSendMbx(renderMessagebox, AMsg);\
	}
	
//Change the fontset the renderer is using
#define renderSetFont(type) {\
	RenderMsg* AMsg = malloc(sizeof(RenderMsg));\
	AMsg->flags=RENDER_SETFONT;\
	AMsg->extra1=type;\
	sceKernelSendMbx(renderMessagebox, AMsg);\
	}
#define FONT_DEFAULT 0
#define FONT_LINEDRAW 1

#define renderDrawCurrentPos(yesno) {\
	RenderMsg* AMsg = malloc(sizeof(RenderMsg));\
	AMsg->flags=RENDER_DRAW_CURRENT_POS;\
	AMsg->extra1=yesno;\
	sceKernelSendMbx(renderMessagebox, AMsg);\
	}

#endif
